#include <nan.h>
#include <X11/Xlib.h>
#include <X11/X.h>
#include <X11/Xutil.h>

#include <CImg.h>

using namespace v8;

Display *display = XOpenDisplay(NULL);

void TakeScreenshot(const Nan::FunctionCallbackInfo<Value>& info) {
	Window root;

	if (info.Length() < 1) {
		Nan::ThrowTypeError("Doesn't have filename argument!");
		return;
	}

	if (info.Length() > 1) {
		Nan::ThrowTypeError("Too many arguments!");
		return;
	}

	if (!info[0]->IsString()) {
		Nan::ThrowTypeError("Argument not a string!");
		return;
	}

	v8::String::Utf8Value param1(info[0]->ToString());

	std::string arg1 = std::string(*param1);

	if (arg1.find(".png") == std::string::npos) {
		arg1 = arg1 + ".png";
	}

	Handle<Value> v8filename = String::NewFromUtf8(Isolate::GetCurrent(), arg1.c_str());

	root = XDefaultRootWindow(display);

	XWindowAttributes gwa;

	XGetWindowAttributes(display, root, &gwa);
	int width = gwa.width;
	int height = gwa.height;

	XImage *image = XGetImage(display, root, 0, 0, width, height, AllPlanes, ZPixmap);

	unsigned long red_mask = image->red_mask;
	unsigned long green_mask = image->green_mask;
	unsigned long blue_mask = image->blue_mask;

	cimg_library::CImg<unsigned char> pic(width, height, 1, 3);
	for (int y = 0; y < height; ++y) {
		for(int x = 0; x < width; ++x) {
			unsigned long pixel = XGetPixel(image, x, y);

			unsigned char blue = pixel & blue_mask;
			unsigned char green = (pixel & green_mask) >> 8;
			unsigned char red = (pixel & red_mask) >> 16;

			pic(x, y, 0) = red;
			pic(x, y, 1) = green;
			pic(x, y, 2) = blue;
		}
	}
	pic.save_png(arg1.c_str());
 
	info.GetReturnValue().Set(v8filename);
}

void TakeWinshot(const Nan::FunctionCallbackInfo<Value>& info) {
	Window win;
	int revert;

	if (info.Length() < 1) {
		Nan::ThrowTypeError("Doesn't have filename argument!");
		return;
	}

	if (info.Length() > 1) {
		Nan::ThrowTypeError("Too many arguments!");
		return;
	}

	if (!info[0]->IsString()) {
		Nan::ThrowTypeError("Argument not a string!");
		return;
	}

	v8::String::Utf8Value param1(info[0]->ToString());

	std::string arg1 = std::string(*param1);

	if (arg1.find(".png") == std::string::npos) {
		arg1 = arg1 + ".png";
	}

	Handle<Value> v8filename = String::NewFromUtf8(Isolate::GetCurrent(), arg1.c_str());

	XGetInputFocus(display, &win, &revert);

	XWindowAttributes gwa;

	XGetWindowAttributes(display, win, &gwa);
	int width = gwa.width;
	int height = gwa.height;

	XImage *image = XGetImage(display, win, 0, 0, width, height, AllPlanes, ZPixmap);

	unsigned long red_mask = image->red_mask;
	unsigned long green_mask = image->green_mask;
	unsigned long blue_mask = image->blue_mask;

	cimg_library::CImg<unsigned char> pic(width, height, 1, 3);
	for (int y = 0; y < height; ++y) {
		for(int x = 0; x < width; ++x) {
			unsigned long pixel = XGetPixel(image, x, y);

			unsigned char blue = pixel & blue_mask;
			unsigned char green = (pixel & green_mask) >> 8;
			unsigned char red = (pixel & red_mask) >> 16;

			pic(x, y, 0) = red;
			pic(x, y, 1) = green;
			pic(x, y, 2) = blue;
		}
	}
	pic.save_png(arg1.c_str());

	info.GetReturnValue().Set(v8filename);
}

void Init(Local<Object> exports) {
	  exports->Set(Nan::New("takeShot").ToLocalChecked(),
               Nan::New<v8::FunctionTemplate>(TakeScreenshot)->GetFunction());
		  exports->Set(Nan::New("takeWinShot").ToLocalChecked(),
               Nan::New<v8::FunctionTemplate>(TakeWinshot)->GetFunction());
}

NODE_MODULE(addon, Init);